/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * An abstract monitoring sub-function managing thread pool.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractThreadPool implements MonitoringSubFunction {

    /**
     * The option to get the component name
     */
    public static final String COMPONENT_NAME_LONG_OPTION = "component-name";

    public static final String COMPONENT_NAME_SHORT_OPTION = "n";

    private static final Option COMPONENT_NAME_OPTION = OptionBuilder.isRequired(true).hasArgs(1)
            .withArgName("component-name").withDescription("The identifier of the component.")
            .withLongOpt(COMPONENT_NAME_LONG_OPTION).create(COMPONENT_NAME_SHORT_OPTION);

    /**
     * Header of the metric value associated to the max allocated threads of the
     * thread pool
     */
    public static final String HEADER_ALLOCATED_THREAD_MAX = "AllocatedThreadsMax";

    /**
     * Header of the metric value associated to the current allocated threads of
     * the thread pool
     */
    public static final String HEADER_ALLOCATED_THREAD_CURRENT = "AllocatedThreadsCurrent";

    /**
     * Header of the metric value associated to the max idle threads of the
     * thread pool
     */
    public static final String HEADER_IDLE_THREAD_MAX = "IdleThreadsMax";

    /**
     * Header of the metric value associated to the current idle threads of the
     * thread pool
     */
    public static final String HEADER_IDLE_THREAD_CURRENT = "IdleThreadsCurrent";

    /**
     * Header of the metric value associated to the max size of the thread pool
     */
    public static final String HEADER_MAX_SIZE = "MaxSize";

    /**
     * Header of the metric value associated to the min size of the thread pool
     */
    public static final String HEADER_MIN_SIZE = "MinSize";

    /**
     * Header of the metric value associated to the current enqueued request
     * number in the thread pool
     */
    public static final String HEADER_ENQUEUED_REQUEST_CURRENT = "EnqueuedRequestsCurrent";

    /**
     * Header of the metric value associated to the max enqueued request number
     * in the HTTP server thread pool
     */
    public static final String HEADER_ENQUEUED_REQUEST_MAX = "EnqueuedRequestsMax";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public AbstractThreadPool() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(COMPONENT_NAME_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    /**
     * Get the current metrics of the thread pool of the component
     * 
     * @param client
     *            The JMX client to use to retrieve current metrics of the
     *            thread pool
     * @param componenIdentifier
     *            Identifier of the component
     * @return The current metrics of the thread pool
     * @throws ComponentMonitoringServiceDoesNotExistException
     *             The monitoring service needed to retrieve current metrics of
     *             the thread pool does not exist.
     * @throws ComponentMonitoringServiceErrorException
     *             An error occurs retrieving the current metrics of the thread
     *             pool.
     * @throws ConnectionErrorException
     *             A connection error occurs
     * 
     */
    protected abstract ThreadPoolMetrics getThreadPoolMetrics(final JMXClient client,
            final String componentIdentifier)
            throws ComponentMonitoringServiceDoesNotExistException,
            ComponentMonitoringServiceErrorException, ConnectionErrorException;

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = JMXClientConnection.getJMXClient();
            try {

                final ThreadPoolMetrics threadPoolMetrics = this.getThreadPoolMetrics(client,
                        componentIdentifier);

                final StringBuilder reply = new StringBuilder();
                final Long allocatedThreadsMax = threadPoolMetrics.getAllocatedThreadsMaxSize();
                if (allocatedThreadsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_ALLOCATED_THREAD_MAX,
                            allocatedThreadsMax.longValue()));
                }

                final Long allocatedThreadsCurrent = threadPoolMetrics.getAllocatedThreadsSize();
                if (allocatedThreadsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_ALLOCATED_THREAD_CURRENT,
                            allocatedThreadsCurrent.longValue()));
                }

                final Long idleThreadsMax = threadPoolMetrics.getIdleThreadsMaxSize();
                if (idleThreadsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_THREAD_MAX,
                            idleThreadsMax.longValue()));
                }

                final Long idleThreadsCurrent = threadPoolMetrics.getIdleThreadsSize();
                if (idleThreadsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_THREAD_CURRENT,
                            idleThreadsCurrent.longValue()));
                }

                final Long maxSize = threadPoolMetrics.getMaxSize();
                if (maxSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MAX_SIZE, maxSize.longValue()));
                }

                final Long minSize = threadPoolMetrics.getMinSize();
                if (minSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MIN_SIZE, minSize.longValue()));
                }

                final Long enqueuedRequestsCurrent = threadPoolMetrics.getEnqueuedRequestsSize();
                if (enqueuedRequestsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_ENQUEUED_REQUEST_CURRENT,
                            enqueuedRequestsCurrent.longValue()));
                }

                final Long enqueuedRequestsMax = threadPoolMetrics.getEnqueuedRequestsMaxSize();
                if (enqueuedRequestsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_ENQUEUED_REQUEST_MAX,
                            enqueuedRequestsMax.longValue()));
                }

                this.shell.getPrintStream().println(reply.toString());

            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier));
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
