/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;

/**
 * The monitoring sub-function about incoming requests counters of the Petals BC
 * SOAP.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingHttpRequestsCount implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "incoming-http-requests-count";

    private final static String SUBFCT_DESCRIPTION = "Counters of incoming HTTP requests";

    /**
     * The option to get the component name
     */
    public static final String COMPONENT_NAME_LONG_OPTION = "component-name";

    public static final String COMPONENT_NAME_SHORT_OPTION = "n";

    private static final Option COMPONENT_NAME_OPTION = OptionBuilder.isRequired(true).hasArgs(1)
            .withArgName("component-name").withDescription("The identifier of the BC SOAP.")
            .withLongOpt(COMPONENT_NAME_LONG_OPTION).create(COMPONENT_NAME_SHORT_OPTION);

    /**
     * The option to get statistics about incoming requests by their nature
     * (welcome servlet, service list servlet, unknown request, ws-request).
     */
    public static final String REQUEST_NATURE_SHORT_OPTION = "r";

    private static final Option REQUEST_NATURE_OPTION = OptionBuilder.isRequired(false)
            .withDescription("Get consolidate values by request natures.")
            .create(REQUEST_NATURE_SHORT_OPTION);

    /**
     * Header of the metric value associated to the information HTTP request
     * counter
     */
    protected final static String LABEL_INFORMATION = "Information";

    /**
     * Header of the metric value associated to the HTTP ws-request counter
     */
    protected final static String LABEL_WS = "WS";

    /**
     * Header of the metric value associated to the unknown HTTP request counter
     */
    protected final static String LABEL_UNKNOWN = "Unknown";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public IncomingHttpRequestsCount() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [-r]
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(COMPONENT_NAME_OPTION);
        this.options.addOption(REQUEST_NATURE_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(REQUEST_NATURE_SHORT_OPTION)) {

                try {
                    final JMXClient client = JMXClientConnection.getJMXClient();
                    final PetalsBcSoapMonitoringServiceClient bcSoapClient = client
                            .getPetalsBcSoapMonitoringServiceClient(componentIdentifier);

                    final StringBuilder reply = new StringBuilder();
                    final Long information = bcSoapClient.getInformationHttpRequestsCount();
                    if (information != null) {
                        reply.append(String.format("%s:%d ", LABEL_INFORMATION,
                                information.longValue()));
                    }
                    final Long unknown = bcSoapClient.getUnknownHttpRequestsCount();
                    if (unknown != null) {
                        reply.append(String.format("%s:%d ", LABEL_UNKNOWN, unknown.longValue()));
                    }
                    final Long ws = this.getConsolidatedValues(componentIdentifier, null, null,
                            null);
                    if (ws != null) {
                        reply.append(String.format("%s:%d ", LABEL_WS, ws.longValue()));
                    }

                    this.shell.getPrintStream().println(reply.toString());

                } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                    throw new MonitoringSubFunctionException(this, String.format(
                            Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier));
                }

            } else {
                final List<String> missingOptions = new ArrayList<String>();
                missingOptions.add(IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION);
                throw new MonitoringSubFunctionMissingOptionsException(this, missingOptions);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * Get the consolidated value about WS request counters matching provided
     * regexp
     * 
     * @param componentIdentifier
     *            The identifier of the BC SOAP
     * @param serviceRegexp
     *            The regexp of services for which the consolidation must be
     *            done
     * @param clientRegexp
     *            The regexp of client for which the consolidation must be done
     * @param execStatusRegexp
     *            Execution status filter. If <code>null</code>, no filter
     *            applied.
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     * @return The consolidated value matching provided regexp, or
     *         <code>null</code> if no values is available.
     */
    private Long getConsolidatedValues(final String componentIdentifier,
            final String serviceRegexp, final String clientRegexp, final String execStatusRegexp)
            throws MonitoringSubFunctionException {

        try {

            final Pattern serviceNamePattern;
            try {
                serviceNamePattern = Pattern.compile(serviceRegexp == null ? ".*" : serviceRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", serviceRegexp), e);
            }

            final Pattern clientPattern;
            try {
                clientPattern = Pattern.compile(clientRegexp == null ? ".*" : clientRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", clientRegexp), e);
            }

            final Pattern execStatusPattern;
            try {
                execStatusPattern = Pattern.compile(execStatusRegexp == null ? ".*"
                        : execStatusRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        "Invalid regexp: '%s'", execStatusRegexp), e);
            }

            final JMXClient client = JMXClientConnection.getJMXClient();
            final Map<IncomingWsRequestKey, Long> values = client
                    .getPetalsBcSoapMonitoringServiceClient(
                    componentIdentifier).getIncomingWsRequestsCount();

            if (values != null) {
                long consolidatedValue = 0;
                for (final Map.Entry<IncomingWsRequestKey, Long> entry : values.entrySet()) {

                    final IncomingWsRequestKey key = entry.getKey();
                    final String wsPath = key.getWsPath();
                    final Matcher serviceNameMatcher = serviceNamePattern
                            .matcher(wsPath == null ? "" : wsPath);
                    final String clientIpAddress = key.getClientIpAddress();
                    final Matcher clientMatcher = clientPattern
                            .matcher(clientIpAddress == null ? "" : clientIpAddress);
                    final String execStatus = key.getExecStatus().toString();
                    final Matcher execStatusMatcher = execStatusPattern
                            .matcher(execStatus == null ? "" : execStatus);

                    if (serviceNameMatcher.matches() && clientMatcher.matches()
                            && execStatusMatcher.matches()) {
                        consolidatedValue += entry.getValue().longValue();
                    }
                }

                return Long.valueOf(consolidatedValue);
            }
 else {
                return null;
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
