/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE;

import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.util.SimpleShell;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link MexProcessorObjectPool}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexProcessorObjectPoolTest extends AbstractTestCase {

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        try {
            monitoringSubFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        try {
            monitoringSubFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>error occurs getting HTTP thread pool value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_01() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max
     * borrowed objects of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max borrowed objects of message
     * exchange processor object pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = null;
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current
     * borrowed objects of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current borrowed objects of
     * message exchange processor object pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_03() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = null;
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertFalse(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current
     * idle objects of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current idle objects of message
     * exchange processor object pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_04() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = null;
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertFalse(
                "the metric value associated to the current idle objects of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max idle
     * objects of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max idle objects of message
     * exchange processor object pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_05() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = null;
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertFalse(
                "the metric value associated to the max idle objects of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the exhaustion
     * number of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the exhaustion number of message
     * exchange processor object pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = null;
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is ot displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertFalse(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max size of
     * message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max size of message exchange
     * processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = null;
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertFalse(
                "the metric value associated to the max size of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min idle size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the min idle
     * size of message exchange processor object pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the min idle size of message exchange
     * processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long.valueOf(123);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = null;
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(123);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current borrowed objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX));
        assertTrue(
                "the metric value associated to the current idle objects of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_MAX_SIZE));
        assertFalse(
                "the metric value associated to the min idle size of message exchange processor object pool is displayed",
                output.contains(MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE));
        assertTrue(
                "the metric value associated to the exhaustion number in the message exchange processor object pool is not displayed",
                output.contains(MexProcessorObjectPool.HEADER_EXHAUSTIONS));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new MexProcessorObjectPool();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long mexProcessorObjectPoolBorrowedObjectCurrent = 123;
        final long mexProcessorObjectPoolBorrowedObjectMax = 167;
        final long mexProcessorObjectPoolIdleObjectCurrent = 2;
        final long mexProcessorObjectPoolIdleObjectMax = 7;
        final long mexProcessorObjectPoolExhaustions = 1;
        final long mexProcessorObjectPoolMinSize = 5;
        final long mexProcessorObjectPoolMaxSize = 25;
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = Long
                .valueOf(mexProcessorObjectPoolBorrowedObjectCurrent);
        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = Long
                .valueOf(mexProcessorObjectPoolBorrowedObjectMax);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = Long
                .valueOf(mexProcessorObjectPoolIdleObjectCurrent);
        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = Long
                .valueOf(mexProcessorObjectPoolIdleObjectMax);
        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = Long
.valueOf(mexProcessorObjectPoolExhaustions);
        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = Long.valueOf(mexProcessorObjectPoolMinSize);
        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = Long.valueOf(mexProcessorObjectPoolMaxSize);
        monitoringSubFunction.execute(new String[] {
                "-" + MexProcessorObjectPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingMexProcessorObjectPoolBorrowedObjectCurrent = true;
        boolean isMissingMexProcessorObjectPoolBorrowedObjectMax = true;
        boolean isMissingMexProcessorObjectPoolIdleObjectCurrent = true;
        boolean isMissingMexProcessorObjectPoolIdleObjectMax = true;
        boolean isMissingMexProcessorObjectPoolExhaustions = true;
        boolean isMissingMexProcessorObjectPoolMinIdleSize = true;
        boolean isMissingMexProcessorObjectPoolMaxSize = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(),
                    ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_CURRENT
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current borrowed objects of message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT,
                        Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolBorrowedObjectCurrent = false;
            } else if (MexProcessorObjectPool.HEADER_BORROWED_OBJECTS_MAX
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max borrowed objects of message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX, Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolBorrowedObjectMax = false;
            } else if (MexProcessorObjectPool.HEADER_IDLE_OBJECTS_MAX
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max idle objects of message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX, Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolIdleObjectMax = false;
            } else if (MexProcessorObjectPool.HEADER_IDLE_OBJECTS_CURRENT
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current idle objects of message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT, Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolIdleObjectCurrent = false;
            } else if (MexProcessorObjectPool.HEADER_EXHAUSTIONS
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current enqueued request into the message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS,
                        Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolExhaustions = false;
            } else if (MexProcessorObjectPool.HEADER_MIN_IDLE_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the min size of the message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE, Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolMinIdleSize = false;
            } else if (MexProcessorObjectPool.HEADER_MAX_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max size of the message exchange processor object pool",
                        MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE, Long.valueOf(metricValue));
                isMissingMexProcessorObjectPoolMaxSize = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current borrowed objects of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolBorrowedObjectCurrent);
        assertFalse(
                "The metric value associated to the max borrowed objects of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolBorrowedObjectMax);
        assertFalse(
                "The metric value associated to the current idle objects of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolIdleObjectCurrent);
        assertFalse(
                "The metric value associated to the max idle objects of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolIdleObjectMax);
        assertFalse(
                "The metric value associated to the exhaustions of the message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolExhaustions);
        assertFalse(
                "The metric value associated the max size of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolMaxSize);
        assertFalse(
                "The metric value associated the min idle size of message exchange processor object pool is missing",
                isMissingMexProcessorObjectPoolMinIdleSize);
    }
}
