/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOKATIONS_COUNT;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.util.SimpleShell;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationStatus;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link ServiceProviderInvokationsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvokationsCountTest extends AbstractTestCase {

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] { "-"
                    + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                            "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION,
                            "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION,
                    "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvokationsCount.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION,
                    "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        final String invalidArg = "invalid-arg";
        try {
            monitoringSubFunction.execute(new String[] {
                            "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                            invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvokationsCount.QUERY_ATTR_NAME,
                    "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException, ConnectionErrorException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException, ConnectionErrorException {

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_COUNT = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException, ConnectionErrorException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_COUNT = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_NAME });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        SERVICE_PROVIDER_INVOKATIONS_COUNT = null;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.TABLE_SHORT_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long serviceProviderInvokationsCount1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final long serviceProviderInvokationsCount2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1, "mep",
                ServiceProviderInvokationStatus.SUCCEEDED), serviceProviderInvokationsCount1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2, "mep",
                ServiceProviderInvokationStatus.SUCCEEDED), serviceProviderInvokationsCount2);

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvokationKey, Long> entry : SERVICE_PROVIDER_INVOKATIONS_COUNT
                    .entrySet()) {
                final ServiceProviderInvokationKey key = entry.getKey();
                if (line.equals(key.getOperation() + "@" + key.getServiceName() + "@"
                        + key.getInterfaceName())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", SERVICE_PROVIDER_INVOKATIONS_COUNT.size(),
                nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationsOperation1 + "@"
                + serviceProviderInvokationsSvcName1,
                Long.valueOf(outgoingWsRequestCountSucceeded1));
        expectedResults.put(
                serviceProviderInvokationsOperation2 + "@" + serviceProviderInvokationsSvcName2,
                Long.valueOf(serviceProviderInvokationsCountSucceeded2
                        + serviceProviderInvokationsCountSucceeded3));

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_NAME });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvokationKey, Long> entry : SERVICE_PROVIDER_INVOKATIONS_COUNT
                    .entrySet()) {
                final ServiceProviderInvokationKey key = entry.getKey();
                if (operationNameReturned.equals(key.getOperation() + "@" + key.getServiceName()
                        + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_11() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationsOperation1 + "@"
                + serviceProviderInvokationsSvcName1 + "@" + serviceProviderInvokationsItfName1,
                Long.valueOf(outgoingWsRequestCountSucceeded1));
        expectedResults.put(
                serviceProviderInvokationsOperation2 + "@" + serviceProviderInvokationsSvcName2
                        + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountSucceeded2
                        + serviceProviderInvokationsCountSucceeded3));

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_SUCCEEDED });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationsCountOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "SUCCEEDED" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_12() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(
                serviceProviderInvokationsOperation2 + "@" + serviceProviderInvokationsSvcName2
                        + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountSucceeded2
                        + serviceProviderInvokationsCountSucceeded3));

        final String queriedOperation = serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_SUCCEEDED, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_13() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(
                serviceProviderInvokationsOperation1 + "@" + serviceProviderInvokationsSvcName1
                        + "@" + serviceProviderInvokationsItfName1,
                Long.valueOf(serviceProviderInvokationsCountFault1
                        + serviceProviderInvokationsCountFault2));
        expectedResults.put(serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountFault3));

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_FAULT });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationsCountOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "FAULT" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_14() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1,
                serviceProviderInvokationsSvcName1, serviceProviderInvokationsOperation1,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep1, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.FAULT),
                serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2,
                serviceProviderInvokationsSvcName2, serviceProviderInvokationsOperation2,
                serviceProviderInvokationsMep2, ServiceProviderInvokationStatus.ERROR),
                serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountFault3));

        final String queriedOperation = serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_FAULT, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_15() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.ERROR), serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.ERROR), serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationsOperation1 + "@"
                + serviceProviderInvokationsSvcName1 + "@" + serviceProviderInvokationsItfName1,
                Long.valueOf(serviceProviderInvokationsCountError1));
        expectedResults.put(serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountError2));

        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_ERROR });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvokationsCountOutput(output,
                SERVICE_PROVIDER_INVOKATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "ERROR" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_16() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final ByteArrayOutputStream errBAOS = new ByteArrayOutputStream();
        final ByteArrayOutputStream outBAOS = new ByteArrayOutputStream();

        final MonitoringSubFunction monitoringSubFunction = new ServiceProviderInvokationsCount();
        monitoringSubFunction.setShell(new SimpleShell(outBAOS, errBAOS));
        JMXClientConnection.createJMXClient(PetalsJmxApiFactoryMock.HOST_NO_ERROR,
                PetalsJmxApiFactoryMock.DEFAULT_PORT, PetalsJmxApiFactoryMock.DEFAULT_USERNAME,
                PetalsJmxApiFactoryMock.DEFAULT_PASSWORD);
        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvokationsItfName1 = "itfName-1";
        final String serviceProviderInvokationsSvcName1 = "svcName-1";
        final String serviceProviderInvokationsOperation1 = "operation1";
        final String serviceProviderInvokationsMep1 = "mep1";
        final long serviceProviderInvokationsCountSucceeded2 = 10;
        final String serviceProviderInvokationsItfName2 = "itfName-2";
        final String serviceProviderInvokationsSvcName2 = "svcName-2";
        final String serviceProviderInvokationsOperation2 = "operation2";
        final String serviceProviderInvokationsMep2 = "mep2";
        final long serviceProviderInvokationsCountSucceeded3 = 56;
        final long serviceProviderInvokationsCountFault1 = 56;
        final long serviceProviderInvokationsCountFault2 = 12;
        final long serviceProviderInvokationsCountFault3 = 34;
        final long serviceProviderInvokationsCountError1 = 9;
        final long serviceProviderInvokationsCountError2 = 7;

        SERVICE_PROVIDER_INVOKATIONS_COUNT = new HashMap<ServiceProviderInvokationKey, Long>();
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault1);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName1, serviceProviderInvokationsSvcName1,
                serviceProviderInvokationsOperation1, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.ERROR), serviceProviderInvokationsCountError1);

        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep1,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded2);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.SUCCEEDED),
                serviceProviderInvokationsCountSucceeded3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.FAULT), serviceProviderInvokationsCountFault3);
        SERVICE_PROVIDER_INVOKATIONS_COUNT.put(new ServiceProviderInvokationKey(
                serviceProviderInvokationsItfName2, serviceProviderInvokationsSvcName2,
                serviceProviderInvokationsOperation2, serviceProviderInvokationsMep2,
                ServiceProviderInvokationStatus.ERROR), serviceProviderInvokationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<String, Long>();
        expectedResults.put(serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2,
                Long.valueOf(serviceProviderInvokationsCountError2));

        final String queriedOperation = serviceProviderInvokationsOperation2 + "@"
                + serviceProviderInvokationsSvcName2 + "@" + serviceProviderInvokationsItfName2;
        monitoringSubFunction.execute(new String[] {
                "-" + ServiceProviderInvokationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvokationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvokationsCount.QUERY_ATTR_ERROR, queriedOperation });

        final String error = errBAOS.toString();
        final String output = outBAOS.toString();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }
}
