/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.local;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Ignore;
import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;

/**
 * Unit tests of {@link DeliveredMessages}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeliveredMessagesTest {

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {});
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set but without arguments</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] { "--"
                    + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set but without arguments</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] { "--"
                    + DeliveredMessages.QUERY_SERVICES_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_SERVICES_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set but without arguments</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] { "--"
                    + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_INTERFACES_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set but with an invalid value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        final String invalidValue = "invalid-value";
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION, invalidValue });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION, e.getOption().getOpt());
            assertEquals("Unexpected invalid value raises the error", invalidValue, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set but with an invalid value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        final String invalidValue = "invalid-value";
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION, invalidValue });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_SERVICES_LONG_OPTION, e.getOption().getOpt());
            assertEquals("Unexpected invalid value raises the error", invalidValue, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set but with an invalid value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        final String invalidValue = "invalid-value";
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION, invalidValue });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("Unexpected option raises the error",
                    DeliveredMessages.QUERY_INTERFACES_LONG_OPTION, e.getOption().getOpt());
            assertEquals("Unexpected invalid value raises the error", invalidValue, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoint is set to get index name.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_00() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get index name.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_01() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get index name.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_02() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set to get index name,</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_NAME, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get index name,</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_NAME, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get index name,</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_NAME, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoint is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_03() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_04() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_05() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoint is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_06() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_07() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_08() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoint is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_09() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_10() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_11() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        monitoringSubFunction.execute(new String[] {
                "--" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR });
        // TODO: Add an assert about result
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_11() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_12() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Active'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_13() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ACTIVE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_14() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_15() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Done'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_16() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_DONE, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying endpoints is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_17() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_ENDPOINTS_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying services is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_18() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_SERVICES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option querying interfaces is set to get number of delivered
     * message with message exchange status 'Error'.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    // TODO: Create a mock implementation of petals-jmx and the following test
    // could be enabled
    @Test
    @Ignore(value = "Ignore this unit test until the JMX connection cannot be mocked")
    public void testExecute_Error_19() throws MonitoringSubFunctionException {

        final MonitoringSubFunction monitoringSubFunction = new DeliveredMessages();
        try {
            monitoringSubFunction.execute(new String[] {
                    "-" + DeliveredMessages.QUERY_INTERFACES_LONG_OPTION,
                    DeliveredMessages.QUERY_ATTR_MSG_STATUS_ERROR, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }
}
